@@ -1,3 +1,4 @@
+import sys
 import logging
 import requests
 from collections import defaultdict
@@ -6,6 +7,13 @@ from octodns.provider.base import BaseProvider
 from octodns.record import Record
 
 single_types = {'ALIAS', 'CNAME', 'PTR'}
+dot_types = single_types | {'NS'}
+no_dot_types = {'A', 'AAAA', 'TXT'}
+dot_fields = {
+    'target_name', 'canonical', 
+    'nameserver', 'ptrdname', 
+    'mail_exchanger', 'target',
+}
 type_map = {
     'A': 'ipv4addr',
     'AAAA': 'ipv6addr',
@@ -43,7 +51,10 @@ type_map = {
 class InfoBlox(requests.Session):
     """Encapsulates all traffic with the InfoBlox WAPI"""
 
-    def __init__(self, fqdn, username, password, verify=True, apiver=None, dns_view=None, network_view=None, log=None):
+    def __init__(
+        self, fqdn, username, password, verify=True, apiver=None,
+        dns_view=None, network_view=None, log_change=False, log=None
+    ):
         super(InfoBlox, self).__init__()
         self.fqdn = fqdn
         self.auth = (username, password)
@@ -51,26 +62,31 @@ class InfoBlox(requests.Session):
         self.network_view = network_view
         self.verify = verify
         self.apiver = apiver or '1.0'
+        self.log_change = log_change
         self.log = log
         if not apiver:
-            self.get_api_version()
+            self.apiver = self.get_api_version()
 
     def url(self, url):
-        return url if url.startswith('https://') else 'https://{0.fqdn}/wapi/v{0.apiver}/{1}'.format(self, url)
+        return f'https://{self.fqdn}/wapi/v{self.apiver}/{url}'
 
     def request(self, method, url, **kwargs):
-        ret = super(InfoBlox, self).request(method, self.url(url),**kwargs)
+        if self.log_change and method not in ('GET', 'HEAD'):
+            self.log.info(f'{method} {url} {kwargs}')
+        ret = super().request(method, self.url(url),**kwargs)
         try:
             ret.raise_for_status()
         except requests.HTTPError:
-            self.log.error('InfoBlox.request: %d %s %s %r %s', ret.status_code, method, url, kwargs, ret.text)
+            self.log.error(
+                'InfoBlox.request: %d %s %s %r %s',
+                ret.status_code, method, url, kwargs, ret.text)
             raise
         return ret
 
     def get_api_version(self):
-        res = self.get('?_schema')
-        self.apiver = '.'.join(map(str,sorted(list(map(int,v.split('.'))) for v in res.json()['supported_versions'])[-1]))
-        return self.apiver
+        vers = self.get('?_schema').json()['supported_versions']
+        vers = ([int(i) for i in v.split('.')] for v in vers)
+        return '.'.join(str(i) for i in sorted(vers)[-1])
 
     def get_zone(self, zone):
         return self.get('zone_auth', params={
@@ -82,7 +98,7 @@ class InfoBlox(requests.Session):
     def get_records(self, type, fields, zone, default_ttl, **extra):
         ret = self.get('record:{0}'.format(type.lower()), params={
             'zone': zone.rstrip('.'), **extra,
-            '_return_fields+': ','.join((() if type == 'NS' else ('ttl','use_ttl') ) + fields),
+            '_return_fields+': ','.join((() if type == 'NS' else ('ttl','use_ttl')) + fields),
             '_paging': 1, '_max_results': 1000, '_return_as_object': 1,
             **({'view': self.dns_view} if self.dns_view else {}),
         }).json()
@@ -93,22 +109,31 @@ class InfoBlox(requests.Session):
         dd = defaultdict(list)
         for d in data:
             dd[d['name']].append(d)
-        return [(rl[0]['ttl'] if rl[0]['use_ttl'] else default_ttl, n, [{
-            k: v for k, v in r.items() if k in fields
+        return [(rl[0]['ttl'] if type != 'NS' and rl[0]['use_ttl'] else default_ttl, n, [{
+            k: (v+'.' if k in dot_fields else v) for k, v in r.items() if k in fields
         } for r in rl], rl) for n, rl in dd.items() if rl]
 
-    def add_record(self, type, zone, name, **fields):
+    def payload_value(self, type, value, ttl, default_ttl):
+        spec = type_map[type]
+        single_field = isinstance(spec, str)
+        return {
+            **(
+                {spec: value[:-1]} if type in dot_types else
+                {spec: value} if single_field else
+                {vk: (getattr(value, k)[:-1] if vk in dot_fields else getattr(value, k)) for vk, k in spec.items()}
+            ),
+            **({} if type == 'NS' else {'use_ttl': ttl != default_ttl, 'ttl': ttl,}),
+        }
+
+    def add_record(self, type, zone, name, value, ttl, default_ttl):
         self.post(f'record:{type.lower()}', json={
             'name': f'{name}.{zone}',
+            **self.payload_value(type, value, ttl, default_ttl),
             **({'view': self.dns_view} if self.dns_view else {}),
-            **fields
         })
 
-    def mod_record(self, src, ttl, default_ttl):
-        self.put(src['_ref'], json={
-            **{k: v for k, v in src.items() if k != '_ref'},
-            **({} if type == 'NS' else {'use_ttl': ttl != default_ttl, 'ttl': ttl,}),
-        })
+    def mod_record(self, type, src, value, ttl, default_ttl):
+        self.put(src['_ref'], json=self.payload_value(type, value, ttl, default_ttl))
 
     def del_record(self, source):
         for src in source:
@@ -119,18 +144,16 @@ class InfoBloxProvider(BaseProvider):
 
     SUPPORTS_GEO = False
     SUPPORTS_DYNAMIC = False
-    # SUPPORTS = {*type_map}
 
     def __init__(
         self, id, gridmaster, username, password, verify=True, apiver=None,
-        dns_view=None, network_view=None, *args, **kwargs
+        dns_view=None, network_view=None, log_change=False, *args, **kwargs
     ):
-        self.log = logging.getLogger('{}[{}]'.format(
-            self.__class__.__name__, id))
-        self.conn = InfoBlox(gridmaster, username, password, verify, apiver, dns_view, network_view, self.log)
-        self.log.debug(
-            '__init__: id=%s, gridmaster=%s, username=%s, apiver=%s',
-            id, gridmaster, username, self.conn.apiver)
+        self.log = logging.getLogger(f'{self.__class__.__name__}[{id}]')
+        self.conn = InfoBlox(
+            gridmaster, username, password, verify, apiver, dns_view,
+            network_view, log_change, self.log)
+        self.log.debug(f'__init__: https://{username}@{gridmaster}/wapi/v{self.conn.apiver}/')
         super(InfoBloxProvider, self).__init__(id, *args, **kwargs)
 
     @property
@@ -145,7 +168,7 @@ class InfoBloxProvider(BaseProvider):
         fields = (spec,) if single_field else (*spec,)
         data = self.conn.get_records(type, fields, zone, default_ttl)
         return [(ttl, name, source,
-            values[0][spec]+'.' if type in single_types else [
+            values[0][spec] if type in single_types else [
                 v[spec] if single_field else 
                 {k: v[vk] for vk, k in spec.items()}
                 for v in values
@@ -153,20 +176,6 @@ class InfoBloxProvider(BaseProvider):
         ) for ttl, name, values, source in data]
 
     def populate(self, zone, target=False, lenient=False):
-        '''
-        Loads all records the provider knows about for the provided zone
-
-        When `target` is True the populate call is being made to load the
-        current state of the provider.
-
-        When `lenient` is True the populate call may skip record validation and
-        do a "best effort" load of data. That will allow through some common,
-        but not best practices stuff that we otherwise would reject. E.g. no
-        trailing . or mising escapes for ;.
-
-        When target is True (loading current state) this method should return
-        True if the zone exists or False if it does not.
-        '''
         self.log.debug('populate: name=%s, target=%s, lenient=%s', zone.name,
                        target, lenient)
 
@@ -186,6 +195,7 @@ class InfoBloxProvider(BaseProvider):
             for t, n, s, v in self._data_for(_type, zone.name, default_ttl):
                 record_name = zone.hostname_from_fqdn(n)
                 record = Record.new(zone, record_name, {
+                    # '_refs': s,
                     'ttl': t,
                     'type': _type,
                     'values' if isinstance(v, list) else 'value': v
@@ -197,51 +207,30 @@ class InfoBloxProvider(BaseProvider):
     def _apply_Create(self, zone, change, default_ttl):
         new = change.new
         type = new._type
-        spec = type_map[type]
-        single_field = isinstance(spec, str)
         values = [new.value] if type in single_types else new.values
-        fields = (spec,) if single_field else (*spec,)
         for value in values:
-            self.conn.add_record(type, zone, new.name,
-                ttl=new.ttl,
-                use_ttl=new.ttl!=default_ttl,
-                **(
-                    {spec: value.rstrip('.')} if type in single_types else
-                    {spec: value} if single_field else 
-                    {vk: getattr(value, k) for vk, k in spec.items()}
-                )
-            )
+            self.conn.add_record(type, zone, new.name, value, new.ttl, default_ttl)
 
     def _apply_Delete(self, zone, change, default_ttl):
         self.conn.del_record(change.existing.source)
 
     def _apply_Update(self, zone, change, default_ttl):
-        existing = change.existing
+        ext = change.existing
         new = change.new
         type = new._type
-        update = existing.ttl != new.ttl
-        spec = type_map[type]
-        single_field = isinstance(spec, str)
+        update = ext.ttl != new.ttl
         values = [new.value] if type in single_types else new.values
-        evalues = [existing.value,] if type in single_types else existing.values
-        fields = (spec,) if single_field else (*spec,)
+        evalues = [ext.value,] if type in single_types else ext.values
         for value in values:
             if type in single_types:
-                self.conn.mod_record(existing.source[0], new.ttl, default_ttl)
+                self.conn.mod_record(type, ext.source[0], value, new.ttl, default_ttl)
             elif value in evalues:
                 if update:
-                    self.conn.mod_record(existing.source[evalues.index(value)], new.ttl, default_ttl)
+                    self.conn.mod_record(type, ext.source[evalues.index(value)], value, new.ttl, default_ttl)
             else:
-                self.conn.add_record(type, zone, new.name,
-                    ttl=new.ttl,
-                    use_ttl=new.ttl!=default_ttl,
-                    **(
-                        {spec: value.rstrip('.')} if type in single_types else
-                        {spec: value} if single_field else 
-                        {vk: getattr(value, k) for vk, k in spec.items()}
-                    )
-                )
-        self.conn.del_record(existing.source[i] for i, value in enumerate(evalues) if value not in values)
+                self.conn.add_record(type, zone, new.name, value, new.ttl, default_ttl)
+        if type not in single_types:
+            self.conn.del_record(ext.source[i] for i, value in enumerate(evalues) if value not in values)
 
     def _apply(self, plan):
 
